#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

int SCREEN_WIDTH = 128; // OLED display width, in pixels
int SCREEN_HEIGHT = 64; // OLED display height, in pixels

// Declaration for an SSD1306 display using I2C (SDA, SCL pins)
// Uncomment the following code to use I2C
Adafruit_SSD1306 oled(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);


// Declaration for an SSD1306 display using SPI
// Uncomment the following code to use SPI
//#define OLED_MOSI  9
//#define OLED_CLK   10
//#define OLED_RESET 13
//#define OLED_DC    11
//#define OLED_CS    12
//Adafruit_SSD1306 oled(SCREEN_WIDTH, SCREEN_HEIGHT,
//  OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);


int clockCenterX = 64;
int clockCenterY = 32;

int year;
int month;
int date;
int day;
int hour;
int minute;
int second;

void setTime() {
  year = 2023;
  month = 5;
  date = 31;
  day = 3;
  hour = 2;
  minute = 45;
  second = 30;
}

void getTime() {
  second++;
  if (second == 60) {
    second = 0;
    minute++;
  }
  if (minute == 60) {
    minute = 0;
    hour++;
  }
  if (hour == 24) {
    hour = 0;
    date++;
    day++;
  }
}

void drawHand(float rotation, float ratio, int radius) {
  float angle = 0;
  int X2 = 0;
  int Y2 = 0;
  angle = (rotation-15) * 2.0 * 3.1415 / ratio;
  X2 = clockCenterX + radius * cos(angle);
  Y2 = clockCenterY + radius * sin(angle);
 oled.drawLine(clockCenterX, clockCenterY, X2, Y2, WHITE);
}

void drawClockFace() {
  float angle = 0;
  int X2 = 0;
  int Y2 = 0;
  int X3 = 0;
  int Y3 = 0;
  oled.drawCircle(64, 32, 31.5, WHITE);
  oled.setCursor(58, 5);
  oled.println("12");  
  oled.setCursor(86, 30);
  oled.println("3");
  oled.setCursor(61, 52);
  oled.println("6");
  oled.setCursor(38,30);
  oled.println("9");
  for(int dash = 0; dash<12; dash++) { // draw the "5 minute" dashes
    angle = dash / 12.0 * 2 * 3.1415;
    X2 = clockCenterX + 30 * cos(angle);
    Y2 = clockCenterY + 30 * sin(angle);
    X3 = clockCenterX + 28 * cos(angle);
    Y3 = clockCenterY + 28 * sin(angle);
    oled.drawLine(X2, Y2, X3, Y3, WHITE);
  }
}

void displayTime() {
  oled.clearDisplay();
  drawClockFace();
  drawHand(hour, 12, 15);    
  drawHand(minute, 60, 20);   
  drawHand(second, 60, 25);
  oled.display();
}

void setup() {
  oled.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  delay(2000);
  oled.clearDisplay();

  oled.setTextColor(WHITE);
  oled.clearDisplay();
  setTime();
}

void loop(){
  getTime();
  displayTime();
  delay(1000);
}
